@@ -9,9 +9,10 @@ from typing import Dict, List, Optional, Tuple, Union
 
 from CoreBluetooth import CBCharacteristic
 
-from bleak.backends.characteristic import BleakGATTCharacteristic
-from bleak.backends.corebluetooth.descriptor import BleakGATTDescriptorCoreBluetooth
-from bleak.backends.corebluetooth.utils import cb_uuid_to_str
+from ..characteristic import BleakGATTCharacteristic
+from ..descriptor import BleakGATTDescriptor
+from .descriptor import BleakGATTDescriptorCoreBluetooth
+from .utils import cb_uuid_to_str
 
 
 class CBCharacteristicProperties(Enum):
@@ -57,13 +58,13 @@ _GattCharacteristicsPropertiesEnum: Dict[Optional[int], Tuple[str, str]] = {
 class BleakGATTCharacteristicCoreBluetooth(BleakGATTCharacteristic):
     """GATT Characteristic implementation for the CoreBluetooth backend"""
 
-    def __init__(self, obj: CBCharacteristic):
-        super().__init__(obj)
+    def __init__(self, obj: CBCharacteristic, max_write_without_response_size: int):
+        super().__init__(obj, max_write_without_response_size)
         self.__descriptors: List[BleakGATTDescriptorCoreBluetooth] = []
         # self.__props = obj.properties()
         self.__props: List[str] = [
             _GattCharacteristicsPropertiesEnum[v][0]
-            for v in [2 ** n for n in range(10)]
+            for v in [2**n for n in range(10)]
             if (self.obj.properties() & v)
         ]
         self._uuid: str = cb_uuid_to_str(self.obj.UUID())
@@ -93,13 +94,11 @@ class BleakGATTCharacteristicCoreBluetooth(BleakGATTCharacteristic):
         return self.__props
 
     @property
-    def descriptors(self) -> List[BleakGATTDescriptorCoreBluetooth]:
+    def descriptors(self) -> List[BleakGATTDescriptor]:
         """List of descriptors for this service"""
         return self.__descriptors
 
-    def get_descriptor(
-        self, specifier
-    ) -> Union[BleakGATTDescriptorCoreBluetooth, None]:
+    def get_descriptor(self, specifier) -> Union[BleakGATTDescriptor, None]:
         """Get a descriptor by handle (int) or UUID (str or uuid.UUID)"""
         try:
             if isinstance(specifier, int):
@@ -111,7 +110,7 @@ class BleakGATTCharacteristicCoreBluetooth(BleakGATTCharacteristic):
         except StopIteration:
             return None
 
-    def add_descriptor(self, descriptor: BleakGATTDescriptorCoreBluetooth):
+    def add_descriptor(self, descriptor: BleakGATTDescriptor):
         """Add a :py:class:`~BleakGATTDescriptor` to the characteristic.
 
         Should not be used by end user, but rather by `bleak` itself.
