@@ -6,21 +6,20 @@ import asyncio
 import logging
 import uuid
 import warnings
-from typing import Callable, Optional, Union
-
-from bleak.backends.device import BLEDevice
-from bleak.backends.service import BleakGATTServiceCollection
-from bleak.exc import BleakError
-from bleak.backends.client import BaseBleakClient
-from bleak.backends.p4android.service import BleakGATTServiceP4Android
-from bleak.backends.p4android.characteristic import BleakGATTCharacteristicP4Android
-from bleak.backends.p4android.descriptor import BleakGATTDescriptorP4Android
+from typing import Optional, Set, Union
 
 from android.broadcast import BroadcastReceiver
 from jnius import java_method
 
-from . import defs
-from . import utils
+from ...exc import BleakError
+from ..characteristic import BleakGATTCharacteristic
+from ..client import BaseBleakClient, NotifyCallback
+from ..device import BLEDevice
+from ..service import BleakGATTServiceCollection
+from . import defs, utils
+from .characteristic import BleakGATTCharacteristicP4Android
+from .descriptor import BleakGATTDescriptorP4Android
+from .service import BleakGATTServiceP4Android
 
 logger = logging.getLogger(__name__)
 
@@ -29,21 +28,27 @@ class BleakClientP4Android(BaseBleakClient):
     """A python-for-android Bleak Client
 
     Args:
-        address_or_ble_device (`BLEDevice` or str): The Bluetooth address of the BLE peripheral to connect to or the `BLEDevice` object representing it.
-
-    Keyword Args:
-        disconnected_callback (callable): Callback that will be scheduled in the
-            event loop when the client is disconnected. The callable must take one
-            argument, which will be this client object.
-        adapter (str): Bluetooth adapter to use for discovery. [unused]
+        address_or_ble_device:
+            The Bluetooth address of the BLE peripheral to connect to or the
+            :class:`BLEDevice` object representing it.
+        services:
+            Optional set of services UUIDs to filter.
     """
 
-    def __init__(self, address_or_ble_device: Union[BLEDevice, str], **kwargs):
+    def __init__(
+        self,
+        address_or_ble_device: Union[BLEDevice, str],
+        services: Optional[Set[uuid.UUID]],
+        **kwargs,
+    ):
         super(BleakClientP4Android, self).__init__(address_or_ble_device, **kwargs)
+        self._requested_services = (
+            set(map(defs.UUID.fromString, services)) if services else None
+        )
         # kwarg "device" is for backwards compatibility
         self.__adapter = kwargs.get("adapter", kwargs.get("device", None))
         self.__gatt = None
-        self.__mtu = None
+        self.__mtu = 23
 
     def __del__(self):
         if self.__gatt is not None:
@@ -59,7 +64,7 @@ class BleakClientP4Android(BaseBleakClient):
             Boolean representing connection status.
 
         """
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
 
         self.__adapter = defs.BluetoothAdapter.getDefaultAdapter()
         if self.__adapter is None:
@@ -152,8 +157,7 @@ class BleakClientP4Android(BaseBleakClient):
         self.__callbacks = None
 
         # Reset all stored services.
-        self.services = BleakGATTServiceCollection()
-        self._services_resolved = False
+        self.services = None
 
         return True
 
@@ -167,7 +171,7 @@ class BleakClientP4Android(BaseBleakClient):
             Boolean regarding success of pairing.
 
         """
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
 
         bondedFuture = loop.create_future()
 
@@ -247,21 +251,31 @@ class BleakClientP4Android(BaseBleakClient):
            A :py:class:`bleak.backends.service.BleakGATTServiceCollection` with this device's services tree.
 
         """
-        if self._services_resolved:
+        if self.services is not None:
             return self.services
 
+        services = BleakGATTServiceCollection()
+
         logger.debug("Get Services...")
         for java_service in self.__gatt.getServices():
+            if (
+                self._requested_services is not None
+                and java_service.getUuid() not in self._requested_services
+            ):
+                continue
 
             service = BleakGATTServiceP4Android(java_service)
-            self.services.add_service(service)
+            services.add_service(service)
 
             for java_characteristic in java_service.getCharacteristics():
 
                 characteristic = BleakGATTCharacteristicP4Android(
-                    java_characteristic, service.uuid, service.handle
+                    java_characteristic,
+                    service.uuid,
+                    service.handle,
+                    self.__mtu - 3,
                 )
-                self.services.add_characteristic(characteristic)
+                services.add_characteristic(characteristic)
 
                 for descriptor_index, java_descriptor in enumerate(
                     java_characteristic.getDescriptors()
@@ -273,9 +287,9 @@ class BleakClientP4Android(BaseBleakClient):
                         characteristic.handle,
                         descriptor_index,
                     )
-                    self.services.add_descriptor(descriptor)
+                    services.add_descriptor(descriptor)
 
-        self._services_resolved = True
+        self.services = services
         return self.services
 
     # IO methods
@@ -356,49 +370,10 @@ class BleakClientP4Android(BaseBleakClient):
 
     async def write_gatt_char(
         self,
-        char_specifier: Union[BleakGATTCharacteristicP4Android, int, str, uuid.UUID],
+        characteristic: BleakGATTCharacteristic,
         data: bytearray,
-        response: bool = False,
+        response: bool,
     ) -> None:
-        """Perform a write operation on the specified GATT characteristic.
-
-        Args:
-            char_specifier (BleakGATTCharacteristicP4Android, int, str or UUID): The characteristic to write
-                to, specified by either integer handle, UUID or directly by the
-                BleakGATTCharacteristicP4Android object representing it.
-            data (bytes or bytearray): The data to send.
-            response (bool): If write-with-response operation should be done. Defaults to `False`.
-
-        """
-        if not isinstance(char_specifier, BleakGATTCharacteristicP4Android):
-            characteristic = self.services.get_characteristic(char_specifier)
-        else:
-            characteristic = char_specifier
-
-        if not characteristic:
-            raise BleakError(f"Characteristic {char_specifier} was not found!")
-
-        if (
-            "write" not in characteristic.properties
-            and "write-without-response" not in characteristic.properties
-        ):
-            raise BleakError(
-                f"Characteristic {str(characteristic.uuid)} does not support write operations!"
-            )
-        if not response and "write-without-response" not in characteristic.properties:
-            response = True
-            # Force response here, since the device only supports that.
-        if (
-            response
-            and "write" not in characteristic.properties
-            and "write-without-response" in characteristic.properties
-        ):
-            response = False
-            logger.warning(
-                "Characteristic %s does not support Write with response. Trying without..."
-                % str(characteristic.uuid)
-            )
-
         if response:
             characteristic.obj.setWriteType(
                 defs.BluetoothGattCharacteristic.WRITE_TYPE_DEFAULT
@@ -456,39 +431,17 @@ class BleakClientP4Android(BaseBleakClient):
 
     async def start_notify(
         self,
-        char_specifier: Union[BleakGATTCharacteristicP4Android, int, str, uuid.UUID],
-        callback: Callable[[int, bytearray], None],
+        characteristic: BleakGATTCharacteristic,
+        callback: NotifyCallback,
         **kwargs,
     ) -> None:
-        """Activate notifications/indications on a characteristic.
-
-        Callbacks must accept two inputs. The first will be an integer handle of the characteristic generating the
-        data and the second will be a ``bytearray`` containing the data sent from the connected server.
-
-        .. code-block:: python
-
-            def callback(sender: int, data: bytearray):
-                print(f"{sender}: {data}")
-            client.start_notify(char_uuid, callback)
-
-        Args:
-            char_specifier (BleakGATTCharacteristicP4Android, int, str or UUID): The characteristic to activate
-                notifications/indications on a characteristic, specified by either integer handle,
-                UUID or directly by the BleakGATTCharacteristicP4Android object representing it.
-            callback (function): The function to be called on notification.
         """
-        if not isinstance(char_specifier, BleakGATTCharacteristicP4Android):
-            characteristic = self.services.get_characteristic(char_specifier)
-        else:
-            characteristic = char_specifier
-
-        if not characteristic:
-            raise BleakError(
-                f"Characteristic with UUID {char_specifier} could not be found!"
-            )
-
+        Activate notifications/indications on a characteristic.
+        """
         self._subscriptions[characteristic.handle] = callback
 
+        assert self.__gatt is not None
+
         if not self.__gatt.setCharacteristicNotification(characteristic.obj, True):
             raise BleakError(
                 f"Failed to enable notification for characteristic {characteristic.uuid}"
@@ -559,7 +512,7 @@ class _PythonBluetoothGattCallback(utils.AsyncJavaCallbacks):
             new_state == defs.BluetoothProfile.STATE_DISCONNECTED
             and self._client._disconnected_callback is not None
         ):
-            self._client._disconnected_callback(self._client)
+            self._client._disconnected_callback()
 
     @java_method("(II)V")
     def onMtuChanged(self, mtu, status):
@@ -572,7 +525,7 @@ class _PythonBluetoothGattCallback(utils.AsyncJavaCallbacks):
     @java_method("(I[B)V")
     def onCharacteristicChanged(self, handle, value):
         self._loop.call_soon_threadsafe(
-            self._client._subscriptions[handle], handle, bytearray(value.tolist())
+            self._client._subscriptions[handle], bytearray(value.tolist())
         )
 
     @java_method("(II[B)V")
